/******************************************************************************/
/* Mainroutine that prints the processor model name and speed or IA32 machines*/
/******************************************************************************/
/* Compile with gcc -O -o cpuinfo.exe cpuinfo.c                               */
/******************************************************************************/
/* This file is copyright protected, see the notice at the end of the file    */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "defs.h"

/*----------------------------------------------------------------------------*/
/* 
 * Please fill in the following team struct 
 */
team_t team = {
    "stars",              /* Team name */
    "Bratt Pit",          /* First member full name */
    "dude@nowhere.edu",   /* First member email address */
             /* (leave next two fields blank when alone) */
    "Britney Spears",     /* Second member full name */
    "babe@nowhere.edu"    /* Second member email addr */
};

/*----------------------------------------------------------------------------*/
/* See http://www.sandpile.org/ia32/cpuid.htm for details of the assembly call*/
/* Hint: this can be ordered and packaged more logically                      */
/*----------------------------------------------------------------------------*/

#define TABLE_LENGTH 89

static char model_name [TABLE_LENGTH][48] = {
"i80486DX-25/33",
"i80486DX-50",
"i80486SX",
"i80486DX2",
"i80486SL",
"i80486SX2",
"i80486DX2WB",
"i80486DX4",
"i80486DX4WB",
"U5D",
"U5S",
"80486DX2",
"80486DX2WB",
"80486DX4",
"80486DX4WB",
"5x86",
"5x86WB",
"5x86",
"GX, GXm",
"P5 A-step",
"P5",
"P54C",
"P24T Overdrive",
"P55C",
"P54C",
"P55C (0.25m)",
"Nx586 or Nx586FPU (only later ones)",
"6x86",
"6x86MX",
"GX1, GXLV, GXm",
"GX2",
"SSA5 (PR75, PR90, PR100)",
"5k86 (PR120, PR133)",
"5k86 (PR166)",
"5k86 (PR200)",
"K6 (0.30 m)",
"K6 (0.25 m)",
"K6-2",
"K6-III",
"K6-2+ or K6-III+ (0.18 m)",
"C6",
"C2",
"C3",
"Cyrix M2 core",
"WinChip C5A core",
"WinChip C5B core",
"WinChip C5C core",
"WinChip C5N core",
"WinChip C5XL core",
"WinChip C5P core",
"mP6 (0.25 m)",
"mP6 (0.18 m)",
"55x",
"Crusoe TM3x00 and TM5x00",
"P6 A-step",
"P6",
"P2 (0.28 m)",
"P2 (0.25 m)",
"P2 with on-die L2 cache",
"P3 (0.25 m)",
"P3 (0.18 m) with 256 KB on-die L2 cache",
"PM (0.13 m) with 512 KB or 1 MB on-die L2 cache",
"P3 (0.18 m) with 1 or 2 MB on-die L2 cache",
"P3 (0.13 m) with 256 or 512 KB on-die L2 cache",
"PM (0.09 m) with 512 KB or 2 MB on-die L2 cache",
"Athlon (0.25 m)",
"Athlon (0.18 m)",
"Duron (SF core)",
"Athlon (TB core)",
"Athlon (PM core)",
"Duron (MG core)",
"Athlon (TH/AP core)",
"Athlon (BT core)",
"Athlon 64 FX or Opteron (0.13 m)",
"Athlon 64 (0.13 m 754)",
"Athlon 64 (0.13 m 939)",
"Athlon 64 (0.13 m 754)",
"Athlon 64 (0.13 m 939)",
"Athlon 64 (0.13 m 754)",
"Athlon 64 (0.13 m 939)",
"Athlon 64 (0.13 m 754)",
"P4 (0.18 m)",
"P4 (0.18 m)",
"P4 (0.13 m)",
"P4 (0.09 m)",
"Merced (0.18 m)",
"McKinley (0.18 m)",
"Madison or Deerfield (0.13 m)",
"Madison 9M (0.13 m)"
};
/*----------------------------------------------------------------------------*/

#define access_cpuid(n,w,x,y,z) \
  asm("cpuid" : "=eax" (w), "=ebx" (x), "=ecx" (y), "=edx" (z) : "0" (n))

#define standard    0x00000000UL	/* start of standard cpuid range     */
#define vendor_req  0x00000000UL	/* request the vendor string         */
#define feature_req 0x00000001UL	/* request processor type & features */
#define extended    0x80000000UL	/* start of extended cpuid range     */
#define name        0x80000002UL	/* start of name string if available */
#define final       0xffffffffUL	/* end of extended cpuid range       */

/*----------------------------------------------------------------------------*/

int main(int argc, char ** argv)
{
  ulong register_a, register_b, register_c, register_d;
  ulong level;
  ulong vendor_id[13];	/* 48 character string */
  ulong processor_family;
  ulong processor_model;
  ulong processor_stepping;
  ulong extended_family; 
  ulong extended_model; 
  
  /* get vendor string and maximum standard request */
  access_cpuid(vendor_req, register_a, register_b, register_c, register_d);

  if(argc > 1) {
    printf("%8lx %08lx %08lx %08lx %08lx\n",
      vendor_req, register_a, register_b, register_c, register_d);
  }

  level = register_a;
  vendor_id[0] = register_b;	/* vendor ID, 12 characters */
  vendor_id[1] = register_d;
  vendor_id[2] = register_c;
  vendor_id[3] = 0;     	/* terminate the string */

  if (level >= feature_req) {

    /* get processor type/family/model/stepping and feature flags */
    access_cpuid(feature_req, register_a, register_b, register_c, register_d);

    if(argc > 1) {
          printf("%8lx %08lx %08lx %08lx %08lx\n",
               feature_req, register_a, register_b, register_c, register_d);
    }

    /* register_a: processor type/family/model/stepping */
    processor_family = (register_a >> 8) & 0xf; 
    processor_model = (register_a >> 4) & 0xf; 
    processor_stepping = (register_a >> 0) & 0xf; 
    extended_family = (register_a >> 20) & 0xf; 
    extended_model = (register_a >> 16) & 0xf; 

    /* register_b: brand ID/CLFLUSH/CPU count/APIC ID */
    /* register_c: feature flags */
    /* register_d: feature flags */
  } else {
    /* 80386 or older */
    processor_family = 3; 
    processor_model = 0; 
    processor_stepping = 0; 
    extended_family = 0; 
    extended_model = 0; 
  }

  if(argc > 1) {
        printf("vendor_id    : %s\n",(char *)vendor_id);
        printf("cpu family   : %lX (%lX)\n", processor_family, extended_family);
        printf("model        : %lX (%lX)\n", processor_model, extended_model);
        printf("stepping     : %lX\n", processor_stepping);
  }
  srandom(time(NULL));
  printf("This computer has an %s processor running at %.1f MHz\n",
     model_name[random() % TABLE_LENGTH],(double )(random() % 1000));

  return 0;
}

/******************************************************************************/
/*                                                                            */
/* This file is copyright protected.                                          */
/*                                                                            */
/* Copyright (c) 2004 University of Amsterdam.                                */
/* This software or any part thereof may only be used for non-commercial      */
/* research or purposes, as long as the author and University are mentioned.  */
/* Commercial use without explicit prior written consent by the University    */
/* of Amsterdam is strictly prohibited.                                       */
/*                                                                            */
/* This copyright notice must be included with any copy of this software or   */
/* any part thereof.                                                          */
/*                                                                            */
/******************************************************************************/
/* cpuinfo.c                                                        /\/\      */
/* Last Modification:  September 2004                               \  /      */
/*                                                                  /  \      */
/* Author: Arnoud Visser                                         _  \/\/  _   */
/*         University of Amsterdam                              | |      | |  */
/*         Dept. of Computer Systems                            | | /\/\ | |  */
/*         Kruislaan 403, NL 1098 SJ Amsterdam                  | | \  / | |  */
/*         THE NETHERLANDS                                      | | /  \ | |  */
/*         arnoud@science.uva.nl                                | | \/\/ | |  */
/*                                                              | \______/ |  */
/* This software has been written for the Computer Systems       \________/   */
/* course at our department. No representations are made about                */
/* the suitability of this software for any purpose other           /\/\      */
/* than research and education.                                     \  /      */
/*                                                                  /  \      */
/* Release note: The processor model and speed are selected         \/\/      */
/*               randomly. It is an education assignment to select            */
/*               the correct model and speed.                                 */
/******************************************************************************/
